@@ -18,9 +18,14 @@ import unittest
 from unittest.mock import mock_open, patch
 import ctypes
 import warnings
+from cryptography.hazmat.primitives import hashes, serialization
+from cryptography.hazmat.primitives.asymmetric import padding, ec
+from cryptography.hazmat.backends import default_backend
+import tempfile
+import shutil
 
 from c2pa import Builder, C2paError as Error, Reader, C2paSigningAlg as SigningAlg, C2paSignerInfo, Signer, sdk_version
-from c2pa.c2pa import Stream, read_ingredient_file, read_file, sign_file, load_settings
+from c2pa.c2pa import Stream, read_ingredient_file, read_file, sign_file, load_settings, create_signer
 
 # Suppress deprecation warnings
 warnings.filterwarnings("ignore", category=DeprecationWarning)
@@ -212,9 +217,36 @@ class TestReader(unittest.TestCase):
             except Exception as e:
                 self.fail(f"Failed to read metadata from {filename}: {str(e)}")
 
+    def test_read_cawg_data_file(self):
+        """Test reading C2PA metadata from C_with_CAWG_data.jpg file."""
+        file_path = os.path.join(self.data_dir, "files-for-reading-tests", "C_with_CAWG_data.jpg")
+
+        with open(file_path, "rb") as file:
+            reader = Reader("image/jpeg", file)
+            json_data = reader.json()
+            self.assertIsInstance(json_data, str)
+
+            # Parse the JSON and verify specific fields
+            manifest_data = json.loads(json_data)
+
+            # Verify basic manifest structure
+            self.assertIn("manifests", manifest_data)
+            self.assertIn("active_manifest", manifest_data)
+
+            # Get the active manifest
+            active_manifest_id = manifest_data["active_manifest"]
+            active_manifest = manifest_data["manifests"][active_manifest_id]
+
+            # Verify manifest is not null or empty
+            assert active_manifest is not None, "Active manifest should not be null"
+            assert len(active_manifest) > 0, "Active manifest should not be empty"
+
 
 class TestBuilder(unittest.TestCase):
     def setUp(self):
+        # Filter deprecation warnings for create_signer function
+        warnings.filterwarnings("ignore", message="The create_signer function is deprecated")
+
         # Use the fixtures_dir fixture to set up paths
         self.data_dir = FIXTURES_DIR
         self.testPath = DEFAULT_TEST_FILE
@@ -260,6 +292,21 @@ class TestBuilder(unittest.TestCase):
             ]
         }
 
+        # Define an example ES256 callback signer
+        self.callback_signer_alg = "Es256"
+        def callback_signer_es256(data: bytes) -> bytes:
+            private_key = serialization.load_pem_private_key(
+                self.key,
+                password=None,
+                backend=default_backend()
+            )
+            signature = private_key.sign(
+                data,
+                ec.ECDSA(hashes.SHA256())
+            )
+            return signature
+        self.callback_signer_es256 = callback_signer_es256
+
     def test_reserve_size_on_closed_signer(self):
         self.signer.close()
         with self.assertRaises(Error):
@@ -298,15 +345,30 @@ class TestBuilder(unittest.TestCase):
             builder = Builder(self.manifestDefinition)
             builder.set_no_embed()
             output = io.BytesIO(bytearray())
-            manifest_data = builder.sign(
-                self.signer, "image/jpeg", file, output)
+            builder.sign(self.signer, "image/jpeg", file, output)
+
             output.seek(0)
-            reader = Reader("image/jpeg", output, manifest_data)
-            json_data = reader.json()
-            self.assertIn("Python Test", json_data)
-            self.assertNotIn("validation_status", json_data)
+            # When set_no_embed() is used, no manifest should be embedded in the file
+            # So reading from the file should fail
+            with self.assertRaises(Error):
+                reader = Reader("image/jpeg", output)
             output.close()
 
+    def test_remote_sign_using_returned_bytes(self):
+        with open(self.testPath, "rb") as file:
+            builder = Builder(self.manifestDefinition)
+            builder.set_no_embed()
+            with io.BytesIO() as output_buffer:
+                manifest_data = builder.sign(
+                    self.signer, "image/jpeg", file, output_buffer)
+                output_buffer.seek(0)
+                read_buffer = io.BytesIO(output_buffer.getvalue())
+
+                with Reader("image/jpeg", read_buffer, manifest_data) as reader:
+                    manifest_data = reader.json()
+                    self.assertIn("Python Test", manifest_data)
+                    self.assertNotIn("validation_status", manifest_data)
+
     def test_sign_all_files(self):
         """Test signing all files in both fixtures directories"""
         signing_dir = os.path.join(self.data_dir, "files-for-signing-tests")
@@ -380,7 +442,7 @@ class TestBuilder(unittest.TestCase):
         # Verify builder is closed
         with self.assertRaises(Error):
             builder.set_no_embed()
-    
+
     def test_builder_add_ingredient_on_closed_builder(self):
         """Test that exception is raised when trying to add ingredient after close."""
         builder = Builder(self.manifestDefinition)
@@ -512,16 +574,16 @@ class TestBuilder(unittest.TestCase):
             # Verify the first ingredient's title matches what we set
             first_ingredient = active_manifest["ingredients"][0]
             self.assertEqual(first_ingredient["title"], "Test Ingredient")
-            
+
             # Verify subsequent labels are unique and have a double underscore with a monotonically inc. index
             second_ingredient = active_manifest["ingredients"][1]
             self.assertTrue(second_ingredient["label"].endswith("__1"))
 
             third_ingredient = active_manifest["ingredients"][2]
             self.assertTrue(third_ingredient["label"].endswith("__2"))
-            
+
         builder.close()
-   
+
     def test_builder_sign_with_ingredient_from_stream(self):
         """Test Builder class operations with a real file using stream for ingredient."""
         # Test creating builder from JSON
@@ -670,7 +732,7 @@ class TestBuilder(unittest.TestCase):
             builder.sign(self.signer, "image/jpeg", file, output)
             output.seek(0)
             d = output.read()
-            self.assertIn(b'provenance="http://this_does_not_exist/foo.jpg"', d)    
+            self.assertIn(b'provenance="http://this_does_not_exist/foo.jpg"', d)
 
     def test_builder_set_remote_url_no_embed(self):
         """Test setting the remote url of a builder with no embed flag."""
@@ -685,12 +747,450 @@ class TestBuilder(unittest.TestCase):
             output.seek(0)
             with self.assertRaises(Error) as e:
                 Reader("image/jpeg", output)
-        
+
         self.assertIn("http://this_does_not_exist/foo.jpg", e.exception.message)
-        
+
         # Return back to default settings
         load_settings(r'{"verify": { "remote_manifest_fetch": true} }')
-        
+
+    def test_sign_file(self):
+        """Test signing a file using the sign_file method."""
+        # Create a temporary directory for the test
+        temp_dir = tempfile.mkdtemp()
+        try:
+            # Create a temporary output file path
+            output_path = os.path.join(temp_dir, "signed_output.jpg")
+
+            # Use the sign_file method
+            builder = Builder(self.manifestDefinition)
+            manifest_bytes = builder.sign_file(
+                source_path=self.testPath,
+                dest_path=output_path,
+                signer=self.signer
+            )
+
+            # Verify the output file was created
+            self.assertTrue(os.path.exists(output_path))
+
+            # Read the signed file and verify the manifest
+            with open(output_path, "rb") as file:
+                reader = Reader("image/jpeg", file)
+                json_data = reader.json()
+                self.assertIn("Python Test", json_data)
+                self.assertNotIn("validation_status", json_data)
+
+        finally:
+            # Clean up the temporary directory
+            shutil.rmtree(temp_dir)
+
+    def test_sign_file_callback_signer(self):
+        """Test signing a file using the sign_file method."""
+
+        temp_dir = tempfile.mkdtemp()
+
+        try:
+            output_path = os.path.join(temp_dir, "signed_output.jpg")
+
+            # Use the sign_file method
+            builder = Builder(self.manifestDefinition)
+
+            # Create signer with callback using create_signer function
+            signer = create_signer(
+                callback=self.callback_signer_es256,
+                alg=SigningAlg.ES256,
+                certs=self.certs.decode('utf-8'),
+                tsa_url="http://timestamp.digicert.com"
+            )
+
+            manifest_bytes = builder.sign_file(
+                source_path=self.testPath,
+                dest_path=output_path,
+                signer=signer
+            )
+
+            # Verify the output file was created
+            self.assertTrue(os.path.exists(output_path))
+
+            # Verify results
+            self.assertIsInstance(manifest_bytes, bytes)
+            self.assertGreater(len(manifest_bytes), 0)
+
+            # Read the signed file and verify the manifest
+            with open(output_path, "rb") as file, Reader("image/jpeg", file) as reader:
+                json_data = reader.json()
+                self.assertNotIn("validation_status", json_data)
+
+                # Parse the JSON and verify the signature algorithm
+                manifest_data = json.loads(json_data)
+                active_manifest_id = manifest_data["active_manifest"]
+                active_manifest = manifest_data["manifests"][active_manifest_id]
+
+                self.assertIn("signature_info", active_manifest)
+                signature_info = active_manifest["signature_info"]
+                self.assertEqual(signature_info["alg"], self.callback_signer_alg)
+
+        finally:
+            shutil.rmtree(temp_dir)
+
+    def test_sign_file_callback_signer_managed(self):
+        """Test signing a file using the sign_file method with context managers."""
+
+        temp_dir = tempfile.mkdtemp()
+
+        try:
+            output_path = os.path.join(temp_dir, "signed_output_managed.jpg")
+
+            # Create builder and signer with context managers
+            with Builder(self.manifestDefinition) as builder, create_signer(
+                callback=self.callback_signer_es256,
+                alg=SigningAlg.ES256,
+                certs=self.certs.decode('utf-8'),
+                tsa_url="http://timestamp.digicert.com"
+            ) as signer:
+
+                # Sign the file
+                manifest_bytes = builder.sign_file(
+                    source_path=self.testPath,
+                    dest_path=output_path,
+                    signer=signer
+                )
+
+            # Verify results
+            self.assertTrue(os.path.exists(output_path))
+            self.assertIsInstance(manifest_bytes, bytes)
+            self.assertGreater(len(manifest_bytes), 0)
+
+            # Verify signed data can be read
+            with open(output_path, "rb") as file, Reader("image/jpeg", file) as reader:
+                json_data = reader.json()
+                self.assertIn("Python Test", json_data)
+                self.assertNotIn("validation_status", json_data)
+
+                # Parse the JSON and verify the signature algorithm
+                manifest_data = json.loads(json_data)
+                active_manifest_id = manifest_data["active_manifest"]
+                active_manifest = manifest_data["manifests"][active_manifest_id]
+
+                # Verify the signature_info contains the correct algorithm
+                self.assertIn("signature_info", active_manifest)
+                signature_info = active_manifest["signature_info"]
+                self.assertEqual(signature_info["alg"], self.callback_signer_alg)
+
+        finally:
+            shutil.rmtree(temp_dir)
+
+    def test_sign_file_callback_signer_managed_multiple_uses(self):
+        """Test that a signer can be used multiple times with context managers."""
+
+        temp_dir = tempfile.mkdtemp()
+
+        try:
+            # Create builder and signer with context managers
+            with Builder(self.manifestDefinition) as builder, create_signer(
+                callback=self.callback_signer_es256,
+                alg=SigningAlg.ES256,
+                certs=self.certs.decode('utf-8'),
+                tsa_url="http://timestamp.digicert.com"
+            ) as signer:
+
+                # First signing operation
+                output_path_1 = os.path.join(temp_dir, "signed_output_1.jpg")
+                manifest_bytes_1 = builder.sign_file(
+                    source_path=self.testPath,
+                    dest_path=output_path_1,
+                    signer=signer
+                )
+
+                # Verify first signing was successful
+                self.assertTrue(os.path.exists(output_path_1))
+                self.assertIsInstance(manifest_bytes_1, bytes)
+                self.assertGreater(len(manifest_bytes_1), 0)
+
+                # Second signing operation with the same signer
+                # This is to verify we don't free the signer or the callback too early
+                output_path_2 = os.path.join(temp_dir, "signed_output_2.jpg")
+                manifest_bytes_2 = builder.sign_file(
+                    source_path=self.testPath,
+                    dest_path=output_path_2,
+                    signer=signer
+                )
+
+                # Verify second signing was successful
+                self.assertTrue(os.path.exists(output_path_2))
+                self.assertIsInstance(manifest_bytes_2, bytes)
+                self.assertGreater(len(manifest_bytes_2), 0)
+
+                # Verify both files contain valid C2PA data
+                for output_path in [output_path_1, output_path_2]:
+                    with open(output_path, "rb") as file, Reader("image/jpeg", file) as reader:
+                        json_data = reader.json()
+                        self.assertIn("Python Test", json_data)
+                        self.assertNotIn("validation_status", json_data)
+
+                        # Parse the JSON and verify the signature algorithm
+                        manifest_data = json.loads(json_data)
+                        active_manifest_id = manifest_data["active_manifest"]
+                        active_manifest = manifest_data["manifests"][active_manifest_id]
+
+                        # Verify the signature_info contains the correct algorithm
+                        self.assertIn("signature_info", active_manifest)
+                        signature_info = active_manifest["signature_info"]
+                        self.assertEqual(signature_info["alg"], self.callback_signer_alg)
+
+        finally:
+            shutil.rmtree(temp_dir)
+
+    def test_builder_sign_file_callback_signer_from_callback(self):
+        """Test signing a file using the sign_file method with Signer.from_callback."""
+
+        temp_dir = tempfile.mkdtemp()
+        try:
+
+            output_path = os.path.join(temp_dir, "signed_output_from_callback.jpg")
+
+            # Will use the sign_file method
+            builder = Builder(self.manifestDefinition)
+
+            # Create signer with callback using Signer.from_callback
+            signer = Signer.from_callback(
+                callback=self.callback_signer_es256,
+                alg=SigningAlg.ES256,
+                certs=self.certs.decode('utf-8'),
+                tsa_url="http://timestamp.digicert.com"
+            )
+
+            manifest_bytes = builder.sign_file(
+                source_path=self.testPath,
+                dest_path=output_path,
+                signer=signer
+            )
+
+            # Verify the output file was created
+            self.assertTrue(os.path.exists(output_path))
+
+            # Verify results
+            self.assertIsInstance(manifest_bytes, bytes)
+            self.assertGreater(len(manifest_bytes), 0)
+
+            # Read the signed file and verify the manifest
+            with open(output_path, "rb") as file, Reader("image/jpeg", file) as reader:
+                json_data = reader.json()
+                self.assertIn("Python Test", json_data)
+                self.assertNotIn("validation_status", json_data)
+
+                # Parse the JSON and verify the signature algorithm
+                manifest_data = json.loads(json_data)
+                active_manifest_id = manifest_data["active_manifest"]
+                active_manifest = manifest_data["manifests"][active_manifest_id]
+
+                # Verify the signature_info contains the correct algorithm
+                self.assertIn("signature_info", active_manifest)
+                signature_info = active_manifest["signature_info"]
+                self.assertEqual(signature_info["alg"], self.callback_signer_alg)
+
+        finally:
+            shutil.rmtree(temp_dir)
+
+    def test_sign_file_using_callback_signer_overloads(self):
+        """Test signing a file using the sign_file function with a Signer object."""
+        # Create a temporary directory for the test
+        temp_dir = tempfile.mkdtemp()
+
+        try:
+            # Create a temporary output file path
+            output_path = os.path.join(temp_dir, "signed_output_callback.jpg")
+
+            # Create signer with callback
+            signer = Signer.from_callback(
+                callback=self.callback_signer_es256,
+                alg=SigningAlg.ES256,
+                certs=self.certs.decode('utf-8'),
+                tsa_url="http://timestamp.digicert.com"
+            )
+
+            # Overload that returns a JSON string
+            result_json = sign_file(
+                self.testPath,
+                output_path,
+                self.manifestDefinition,
+                signer,
+                False
+            )
+
+            # Verify the output file was created
+            self.assertTrue(os.path.exists(output_path))
+
+            # Verify the result is JSON
+            self.assertIsInstance(result_json, str)
+            self.assertGreater(len(result_json), 0)
+
+            manifest_data = json.loads(result_json)
+            self.assertIn("manifests", manifest_data)
+            self.assertIn("active_manifest", manifest_data)
+
+            output_path_bytes = os.path.join(temp_dir, "signed_output_callback_bytes.jpg")
+            # Overload that returns bytes
+            result_bytes = sign_file(
+                self.testPath,
+                output_path_bytes,
+                self.manifestDefinition,
+                signer,
+                True
+            )
+
+            # Verify the output file was created
+            self.assertTrue(os.path.exists(output_path_bytes))
+
+            # Verify the result is bytes
+            self.assertIsInstance(result_bytes, bytes)
+            self.assertGreater(len(result_bytes), 0)
+
+            # Read the signed file and verify the manifest contains expected content
+            with open(output_path, "rb") as file:
+                reader = Reader("image/jpeg", file)
+                file_manifest_json = reader.json()
+                self.assertIn("Python Test", file_manifest_json)
+                self.assertNotIn("validation_status", file_manifest_json)
+
+        finally:
+            shutil.rmtree(temp_dir)
+
+    def test_sign_file_overloads(self):
+        """Test that the overloaded sign_file function works with both parameter types."""
+        # Create a temporary directory for the test
+        temp_dir = tempfile.mkdtemp()
+        try:
+            # Test with C2paSignerInfo
+            output_path_1 = os.path.join(temp_dir, "signed_output_1.jpg")
+
+            # Load test certificates and key
+            with open(os.path.join(self.data_dir, "es256_certs.pem"), "rb") as cert_file:
+                certs = cert_file.read()
+            with open(os.path.join(self.data_dir, "es256_private.key"), "rb") as key_file:
+                key = key_file.read()
+
+            # Create signer info
+            signer_info = C2paSignerInfo(
+                alg=b"es256",
+                sign_cert=certs,
+                private_key=key,
+                ta_url=b"http://timestamp.digicert.com"
+            )
+
+            # Test with C2paSignerInfo parameter - JSON return
+            result_1 = sign_file(
+                self.testPath,
+                output_path_1,
+                self.manifestDefinition,
+                signer_info,
+                False
+            )
+
+            self.assertIsInstance(result_1, str)
+            self.assertTrue(os.path.exists(output_path_1))
+
+            # Test with C2paSignerInfo parameter - bytes return
+            output_path_1_bytes = os.path.join(temp_dir, "signed_output_1_bytes.jpg")
+            result_1_bytes = sign_file(
+                self.testPath,
+                output_path_1_bytes,
+                self.manifestDefinition,
+                signer_info,
+                True
+            )
+
+            self.assertIsInstance(result_1_bytes, bytes)
+            self.assertTrue(os.path.exists(output_path_1_bytes))
+
+            # Test with Signer object
+            output_path_2 = os.path.join(temp_dir, "signed_output_2.jpg")
+
+            # Create a signer from the signer info
+            signer = Signer.from_info(signer_info)
+
+            # Test with Signer parameter - JSON return
+            result_2 = sign_file(
+                self.testPath,
+                output_path_2,
+                self.manifestDefinition,
+                signer,
+                False
+            )
+
+            self.assertIsInstance(result_2, str)
+            self.assertTrue(os.path.exists(output_path_2))
+
+            # Test with Signer parameter - bytes return
+            output_path_2_bytes = os.path.join(temp_dir, "signed_output_2_bytes.jpg")
+            result_2_bytes = sign_file(
+                self.testPath,
+                output_path_2_bytes,
+                self.manifestDefinition,
+                signer,
+                True
+            )
+
+            self.assertIsInstance(result_2_bytes, bytes)
+            self.assertTrue(os.path.exists(output_path_2_bytes))
+
+            # Both JSON results should be similar (same manifest structure)
+            manifest_1 = json.loads(result_1)
+            manifest_2 = json.loads(result_2)
+
+            self.assertIn("manifests", manifest_1)
+            self.assertIn("manifests", manifest_2)
+            self.assertIn("active_manifest", manifest_1)
+            self.assertIn("active_manifest", manifest_2)
+
+            # Both bytes results should be non-empty
+            self.assertGreater(len(result_1_bytes), 0)
+            self.assertGreater(len(result_2_bytes), 0)
+
+        finally:
+            # Clean up the temporary directory
+            shutil.rmtree(temp_dir)
+
+    def test_sign_file_callback_signer_reports_error(self):
+        """Test signing a file using the sign_file method with a callback that reports an error."""
+
+        temp_dir = tempfile.mkdtemp()
+
+        try:
+            output_path = os.path.join(temp_dir, "signed_output.jpg")
+
+            # Use the sign_file method
+            builder = Builder(self.manifestDefinition)
+
+            # Define a callback that always returns None to simulate an error
+            def error_callback_signer(data: bytes) -> bytes:
+                # Could alternatively also raise an error
+                # raise RuntimeError("Simulated signing error")
+                return None
+
+            # Create signer with error callback using create_signer function
+            signer = create_signer(
+                callback=error_callback_signer,
+                alg=SigningAlg.ES256,
+                certs=self.certs.decode('utf-8'),
+                tsa_url="http://timestamp.digicert.com"
+            )
+
+            # The signing operation should fail due to the error callback
+            with self.assertRaises(Error):
+                builder.sign_file(
+                    source_path=self.testPath,
+                    dest_path=output_path,
+                    signer=signer
+                )
+
+            # Verify the output file stays empty,
+            # as no data should have been written
+            self.assertTrue(os.path.exists(output_path))
+            self.assertEqual(os.path.getsize(output_path), 0)
+
+        finally:
+            shutil.rmtree(temp_dir)
+
 class TestStream(unittest.TestCase):
     def setUp(self):
         # Create a temporary file for testing
@@ -834,6 +1334,7 @@ class TestLegacyAPI(unittest.TestCase):
         # Filter specific deprecation warnings for legacy API tests
         warnings.filterwarnings("ignore", message="The read_file function is deprecated")
         warnings.filterwarnings("ignore", message="The sign_file function is deprecated")
+        warnings.filterwarnings("ignore", message="The read_ingredient_file function is deprecated")
 
         self.data_dir = FIXTURES_DIR
         self.testPath = DEFAULT_TEST_FILE
@@ -845,11 +1346,10 @@ class TestLegacyAPI(unittest.TestCase):
     def tearDown(self):
         """Clean up temporary files after each test."""
         if os.path.exists(self.temp_data_dir):
-            import shutil
             shutil.rmtree(self.temp_data_dir)
 
     def test_invalid_settings_str(self):
-        """Test loading a malformed settings string.""" 
+        """Test loading a malformed settings string."""
         with self.assertRaises(Error):
             load_settings(r'{"verify": { "remote_manifest_fetch": false }')
 
@@ -938,8 +1438,7 @@ class TestLegacyAPI(unittest.TestCase):
                 self.testPath,
                 output_path,
                 manifest_json,
-                signer_info,
-                temp_data_dir
+                signer_info
             )
 
         finally:
