@@ -1068,109 +1068,6 @@ class TestBuilderWithThreads(unittest.TestCase):
         # Verify all readers completed
         self.assertEqual(active_readers, 0, "Not all readers completed")
 
-    def test_remote_sign_threaded(self):
-        """Test remote signing with multiple threads in parallel"""
-        output1 = io.BytesIO(bytearray())
-        output2 = io.BytesIO(bytearray())
-        sign_errors = []
-        sign_complete = threading.Event()
-        manifest_data1 = None
-        manifest_data2 = None
-
-        def remote_sign(output_stream, manifest_def, thread_id):
-            nonlocal manifest_data1, manifest_data2
-            try:
-                with open(self.testPath, "rb") as file:
-                    builder = Builder(manifest_def)
-                    builder.set_no_embed()
-                    manifest_data = builder.sign(
-                        self.signer, "image/jpeg", file, output_stream)
-                    output_stream.seek(0)
-
-                    # Store manifest data for final verification
-                    if thread_id == 1:
-                        manifest_data1 = manifest_data
-                    else:
-                        manifest_data2 = manifest_data
-
-                    # Verify the signed file
-                    reader = Reader("image/jpeg", output_stream, manifest_data)
-                    json_data = reader.json()
-                    manifest_store = json.loads(json_data)
-                    active_manifest = manifest_store["manifests"][manifest_store["active_manifest"]]
-
-                    # Verify the correct manifest was used
-                    if thread_id == 1:
-                        expected_claim_generator = "python_test_1/0.0.1"
-                        expected_author = "Tester One"
-                    else:
-                        expected_claim_generator = "python_test_2/0.0.1"
-                        expected_author = "Tester Two"
-
-                    self.assertEqual(
-                        active_manifest["claim_generator"],
-                        expected_claim_generator)
-
-                    # Verify the author is correct
-                    assertions = active_manifest["assertions"]
-                    for assertion in assertions:
-                        if assertion["label"] == "com.unit.test":
-                            author_name = assertion["data"]["author"][0]["name"]
-                            self.assertEqual(author_name, expected_author)
-                            break
-
-            except Exception as e:
-                sign_errors.append(f"Thread {thread_id} error: {str(e)}")
-            finally:
-                sign_complete.set()
-
-        # Create and start two threads for concurrent remote signing
-        thread1 = threading.Thread(
-            target=remote_sign,
-            args=(output1, self.manifestDefinition_1, 1)
-        )
-        thread2 = threading.Thread(
-            target=remote_sign,
-            args=(output2, self.manifestDefinition_2, 2)
-        )
-
-        # Start both threads
-        thread1.start()
-        thread2.start()
-
-        # Wait for both threads to complete
-        thread1.join()
-        thread2.join()
-
-        # Check for errors
-        if sign_errors:
-            self.fail("\n".join(sign_errors))
-
-        # Verify the outputs are different before closing
-        output1.seek(0)
-        output2.seek(0)
-        reader1 = Reader("image/jpeg", output1, manifest_data1)
-        reader2 = Reader("image/jpeg", output2, manifest_data2)
-
-        manifest_store1 = json.loads(reader1.json())
-        manifest_store2 = json.loads(reader2.json())
-
-        # Get the active manifests
-        active_manifest1 = manifest_store1["manifests"][manifest_store1["active_manifest"]]
-        active_manifest2 = manifest_store2["manifests"][manifest_store2["active_manifest"]]
-
-        # Verify the manifests are different
-        self.assertNotEqual(
-            active_manifest1["claim_generator"],
-            active_manifest2["claim_generator"])
-        self.assertNotEqual(
-            active_manifest1["title"],
-            active_manifest2["title"])
-
-        # Clean up after verification
-        output1.close()
-        output2.close()
-
     def test_archive_sign_threaded(self):
         """Test archive signing with multiple threads in parallel"""
         archive1 = io.BytesIO(bytearray())
@@ -2096,32 +1993,22 @@ class TestBuilderWithThreads(unittest.TestCase):
         builder.close()
 
     def test_builder_sign_with_multiple_ingredient_random_many_threads(self):
-        """Test Builder class operations with 10 threads, each adding 3 random ingredients and signing a random file."""
+        """Test Builder class operations with 12 threads, each adding 3 specific ingredients and signing a file."""
         # Number of threads to use in the test
-        # We are pushing it here, as we want to test with thread count one to two orders of magnitude
-        # higher than "usual" max numbers of cores on (server) machines may be.
-
         TOTAL_THREADS_USED = 12
 
-        # Get list of files from files-for-reading-tests directory
-        reading_dir = os.path.join(self.data_dir, "files-for-reading-tests")
-
-        # Filter for JPG and PNG files only
-        all_files = [
-            f for f in os.listdir(reading_dir)
-            if os.path.isfile(os.path.join(reading_dir, f))
-            and os.path.splitext(f)[1].lower() in {'.jpg', '.jpeg', '.png'}
+        # Define the specific files to use as ingredients
+        # THose files should be valid to use as ingredient
+        ingredient_files = [
+            os.path.join(self.data_dir, "A_thumbnail.jpg"),
+            os.path.join(self.data_dir, "C.jpg"),
+            os.path.join(self.data_dir, "cloud.jpg")
         ]
 
-        # Ensure we have enough files
-        self.assertGreaterEqual(
-            len(all_files),
-            3,
-            "Need at least 3 JPG/PNG files for testing")
-
         # Thread synchronization
         thread_results = {}
         completed_threads = 0
+        thread_lock = threading.Lock()  # Lock for thread-safe access to shared data
 
         def thread_work(thread_id):
             nonlocal completed_threads
@@ -2129,24 +2016,17 @@ class TestBuilderWithThreads(unittest.TestCase):
                 # Create a new builder for this thread
                 builder = Builder.from_json(self.manifestDefinition)
 
-                # Select 3 random files for ingredients
-                # Use thread_id as seed for reproducibility
-                random.seed(thread_id)
-                ingredient_files = random.sample(all_files, 3)
-
                 # Add each ingredient
-                for i, file_name in enumerate(ingredient_files, 1):
-                    file_path = os.path.join(reading_dir, file_name)
+                for i, file_path in enumerate(ingredient_files, 1):
                     ingredient_json = json.dumps({
-                        "title": f"Thread {thread_id} Ingredient {i} - {file_name}"
+                        "title": f"Thread {thread_id} Ingredient {i} - {os.path.basename(file_path)}"
                     })
 
                     with open(file_path, 'rb') as f:
                         builder.add_ingredient(ingredient_json, "image/jpeg", f)
 
-                # Select a random file for signing
-                sign_file = random.choice(all_files)
-                sign_file_path = os.path.join(reading_dir, sign_file)
+                # Use A.jpg as the file to sign
+                sign_file_path = os.path.join(self.data_dir, "A.jpg")
 
                 # Sign the file
                 with open(sign_file_path, "rb") as file:
@@ -2168,11 +2048,13 @@ class TestBuilderWithThreads(unittest.TestCase):
                     manifest_data = json.loads(json_data)
 
                     # Store results for verification
-                    thread_results[thread_id] = {
-                        'manifest': manifest_data,
-                        'ingredient_files': ingredient_files,
-                        'sign_file': sign_file
-                    }
+                    with thread_lock:
+                        thread_results[thread_id] = {
+                            'manifest': manifest_data,
+                            'ingredient_files': [os.path.basename(f) for f in ingredient_files],
+                            'sign_file': os.path.basename(sign_file_path),
+                            'manifest_hash': hash(json.dumps(manifest_data, sort_keys=True))  # Add hash for comparison
+                        }
 
                     # Clean up streams
                     output.close()
@@ -2181,11 +2063,13 @@ class TestBuilderWithThreads(unittest.TestCase):
                 builder.close()
 
             except Exception as e:
-                thread_results[thread_id] = {
-                    'error': str(e)
-                }
+                with thread_lock:
+                    thread_results[thread_id] = {
+                        'error': str(e)
+                    }
             finally:
-                completed_threads += 1
+                with thread_lock:
+                    completed_threads += 1
 
         # Create and start threads
         threads = []
@@ -2199,26 +2083,31 @@ class TestBuilderWithThreads(unittest.TestCase):
             thread.join()
 
         # Verify all threads completed
-        self.assertEqual(completed_threads, TOTAL_THREADS_USED, f"All {
-                         TOTAL_THREADS_USED} threads should have completed")
+        self.assertEqual(completed_threads, TOTAL_THREADS_USED, f"All {TOTAL_THREADS_USED} threads should have completed")
         self.assertEqual(
             len(thread_results),
             TOTAL_THREADS_USED,
             f"Should have results from all {TOTAL_THREADS_USED} threads")
 
+        # Collect all manifest hashes for comparison
+        manifest_hashes = set()
+        thread_manifest_data = {}
+
         # Verify results for each thread
         for thread_id in range(1, TOTAL_THREADS_USED + 1):
             result = thread_results[thread_id]
 
             # Check if thread encountered an error
             if 'error' in result:
-                self.fail(
-                    f"Thread {thread_id} failed with error: {
-                        result['error']}")
+                self.fail(f"Thread {thread_id} failed with error: {result['error']}")
 
             manifest_data = result['manifest']
             ingredient_files = result['ingredient_files']
-            sign_file = result['sign_file']
+            manifest_hash = result['manifest_hash']
+
+            # Store manifest data for cross-thread comparison
+            thread_manifest_data[thread_id] = manifest_data
+            manifest_hashes.add(manifest_hash)
 
             # Verify active manifest exists
             self.assertIn("active_manifest", manifest_data)
@@ -2234,16 +2123,44 @@ class TestBuilderWithThreads(unittest.TestCase):
             self.assertIsInstance(active_manifest["ingredients"], list)
             self.assertEqual(len(active_manifest["ingredients"]), 3)
 
-            # Verify all ingredients exist with correct thread ID and file
-            # names
-            ingredient_titles = [ing["title"]
-                                 for ing in active_manifest["ingredients"]]
+            # Verify all ingredients exist with correct thread ID and file names
+            ingredient_titles = [ing["title"] for ing in active_manifest["ingredients"]]
             for i, file_name in enumerate(ingredient_files, 1):
-                expected_title = f"Thread {
-                    thread_id} Ingredient {i} - {file_name}"
-                self.assertIn(expected_title, ingredient_titles, f"Thread {
-                              thread_id} should have ingredient with title {expected_title}")
+                expected_title = f"Thread {thread_id} Ingredient {i} - {file_name}"
+                self.assertIn(expected_title, ingredient_titles, f"Thread {thread_id} should have ingredient with title {expected_title}")
 
+            # Verify no cross-thread contamination in ingredient titles
+            for other_thread_id in range(1, TOTAL_THREADS_USED + 1):
+                if other_thread_id != thread_id:
+                    for title in ingredient_titles:
+                        # Check for exact thread ID pattern to avoid false positives
+                        self.assertNotIn(
+                            f"Thread {other_thread_id} Ingredient",
+                            title,
+                            f"Thread {thread_id}'s manifest contains ingredient data from thread {other_thread_id}")
+
+        # Verify all manifests are unique (no data scrambling between threads)
+        self.assertEqual(
+            len(manifest_hashes),
+            TOTAL_THREADS_USED,
+            "Each thread should have a unique manifest (no data scrambling)")
+
+        # Additional verification: Compare manifest structures between threads
+        for thread_id in range(1, TOTAL_THREADS_USED + 1):
+            current_manifest = thread_manifest_data[thread_id]
+
+            # Verify manifest structure is consistent
+            self.assertIn("active_manifest", current_manifest)
+            self.assertIn("manifests", current_manifest)
+
+            # Verify no cross-thread contamination in manifest data
+            for other_thread_id in range(1, TOTAL_THREADS_USED + 1):
+                if other_thread_id != thread_id:
+                    other_manifest = thread_manifest_data[other_thread_id]
+                    self.assertNotEqual(
+                        current_manifest["active_manifest"],
+                        other_manifest["active_manifest"],
+                        f"Thread {thread_id} and {other_thread_id} share the same active manifest ID")
 
 if __name__ == '__main__':
     unittest.main()
